
var Reggie = function()
{
	var reggie = {};
	var internal = {};

	var GENERIC_STRATAGENE_NAME = 'Stratagene.r';
	var EXTERNAL_RNA_NAME = 'External.r';
	
	var CASENAME_WITH_SUFFIX = /^\d{7}C?$/;
	var CASENAME = /^\d{7}$/;
	
	var RCCID_WITH_SUFFIX = /^\d{10}(B|C|D){1}$/;
	
	/*
		Verify that the 'case' name is a 7-digit value + optional 'C' suffix
	*/
	reggie.isValidCaseName = function isValidCaseName(caseName, allowSuffix)
	{
		return caseName.match(allowSuffix ? CASENAME_WITH_SUFFIX : CASENAME);
	}
	
	/*
		Verify that the RCCID number is a 10-digit value + 'B', 'C', or 'D' suffix
	 */
	reggie.isValidRccidNumber = function(rccidNumber)
	{
		return rccidNumber.match(RCCID_WITH_SUFFIX);
	}

	/**
		Check the control digit of the given personal number.
		@pnr The 10-digit personal number in the form YYMMDDXXXX
	*/
	reggie.personalNumberControlDigitCheck = function(pnr)
	{
		var sum = 0;
		var factor = 2;
		for (var i = 0; i < 9; i++)
		{
			var digit = parseInt(pnr.substr(i, 1), 10);
			var tmp = factor * digit;
			sum += tmp >= 10 ? tmp - 9: tmp;
			factor = factor == 2 ? 1 : 2;
		}
		
		var control = 10 - (sum % 10);
		if (control == 10) control = 0;
		return (control == parseInt(pnr.substr(9, 1), 10))
	}
	
	/**
		Autofill a date-string.
		The supplemented date-string will have the format yyyyMMdd when it's returned.
		If the short-date in format MMdd is larger the current date, the year before will be used.
		
		@param shortDate Date in format 'MMdd' or in 'yyMMdd'
		@param allowSixDigitDates Boolean Flag indicating if 6-digit dates should be auto-filled (default = false).
	*/
	reggie.autoFillDate = function(shortDate, allowSixDigitDates)
	{
		shortDate = shortDate.replace("-","");
		var fullDate = shortDate;
	
		var currentDate = new Date();
		var currentYear = currentDate.getFullYear();
		var currentMonth = currentDate.getMonth()+1;
		
		if (shortDate.length == 4 && Dates.isDate(shortDate, 'MMdd'))
		{
			var month = parseInt(shortDate.substring(0, 2), 10);
			var day = parseInt(shortDate.substring(2), 10);
			if ((month > currentMonth) || (month == currentMonth && day > currentDate.getDate())) 
			{
				currentYear--;
			}
			fullDate = currentYear+shortDate;
		}
		else if (allowSixDigitDates && shortDate.length == 6 && Dates.isDate(shortDate, 'yyMMdd'))
		{
			var year = parseInt(shortDate.substring(0,2), 10);		
			fullDate = ((2000+year) > currentYear ? "19" : "20") + fullDate;
		}	
		return fullDate;	
	}
	
	reggie.autoFillTime = function(shortTime)
	{
		var fullTimeString = shortTime;
		if (shortTime.length==3)
		{
			fullTimeString = "0"+shortTime;
		}
		return fullTimeString;
	}

	/**
		Convert a 2-digit year to a 4-digit year. Assuming
		that the short year is between 1 and 100 years old. 
	*/
	reggie.fullYear = function(shortYear)
	{
		var thisYear = new Date().getFullYear();
		var fullYear = shortYear + 100*Math.floor((thisYear - shortYear - 1) / 100);
		return fullYear;
	}
	
	reggie.formatNumber = function(number, unit, baseDecimals)
	{
		if (number == null) return '';
		
		var numDecimals;
		if (!baseDecimals) baseDecimals = 3;
		if (number < 10)
		{
			numDecimals = baseDecimals;
		}
		else if (number < 100)
		{
			numDecimals = baseDecimals-1;
		}
		else
		{
			numDecimals = baseDecimals-2;
		}

		var result = number.toFixed(numDecimals);
		if (unit) result += unit;
		return result;
	}


	
	/**
		Get subtype information.
	*/
	reggie.getSubtypeInfo = function(subtype)
	{
		var url = '../Subtype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSubtypeInfo&name='+subtype;
		var response = Wizard.syncJsonRequest(url);
		return response ? response.subtype : null;
	}
	
	/**
		Load protocols of the given subtype and add them
		to a selection list. This method submits an
		async request to the server and then returns
		immediately.
	*/
	reggie.loadProtocols = function(subtype, list)
	{
		list = Doc.element(list);
		list.length=0;
		list[0] = new Option('loading...', '');
		Doc.addClass(list, 'list-loading');
		var url = '../Protocol.servlet?ID='+App.getSessionId();
		url += '&cmd=GetProtocols&subtype='+subtype;
		Wizard.asyncJsonRequest(url, 
			function(response)
			{
				internal.fillList(response.protocols, list, true);
			}
		);
	}
	
	/**
		Load software of the given subtype and add them
		to a selection list. This method submits an
		async request to the server and then returns
		immediately.
	*/
	reggie.loadSoftware = function(subtype, list)
	{
		list = Doc.element(list);
		list.length=0;
		list[0] = new Option('loading...', '');
		Doc.addClass(list, 'list-loading');
		var url = '../Software.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSoftware&subtype='+subtype;
		Wizard.asyncJsonRequest(url, 
			function(response)
			{
				internal.fillList(response.software, list, true);
			}
		);
	}

	// Fills a selection list with items
	internal.fillList = function(items, list, addNoneOption)
	{
		Doc.removeClass(list, 'list-loading');
		list.length = 0;
		for (var i = 0; i < items.length; i++)
		{
			var item = items[i];
			list[list.length] = new Option(item.name, item.id, item.isDefault);
		}
		if (list.length > 0) Wizard.setInputStatus(list.id, 'valid');
		if (addNoneOption)
		{
			list[list.length] = new Option('- none -', '');
		}
		
	}
	
	reggie.isStratagene = function(sampleName)
	{
		return sampleName.indexOf(GENERIC_STRATAGENE_NAME) == 0;
	}

	reggie.isExternal = function(sampleName)
	{
		return sampleName.indexOf(EXTERNAL_RNA_NAME) == 0;
	}

	
	/**
		Open a popup window and copy a part of the current window to the popup
		and make it suitable for printin.
	
		@param printElementId The id of a html tag in the current page that should
			be copied to the print window
		@param pageTitle The title of the print window
		@param pageOrientation 'landscape' or 'portrait' (default)
		@param printNote Optional note intended to include instructions how to configure
			the browser for optimal printing (eg. set to landscape and scale to 75%)
	*/
	reggie.openPrintWindow = function(printElementId, pageTitle, pageOrientation, printNote, reggieRoot, extraStyles)
	{
		// Default width/height is for 'portrait' orientation
		var width = 900;
		var height = 900;
		if (pageOrientation == 'landscape')
		{
			width = 1300;
			height = 700;
		}
		
		var url = 'print_template.jsp?ID='+App.getSessionId();
		if (extraStyles) url += '&extraStyles='+extraStyles;
		if (reggieRoot) url = reggieRoot+url;
		// Open the print-window 
		internal.printOptions = {
			'printElementId': printElementId,
			'pageTitle': pageTitle, 
			'pageOrientation': pageOrientation, 
			'printNote': printNote
		};
		var printWin = window.open(url, 'PrintWindow', 'width='+width+',height='+height+',toolbar=yes,location=no,directories=no,status=no,menubar=yes,scrollbars=yes,resizable=yes');
	}

	reggie.finalizePrint = function(printWin)
	{
		// Set page title...
		if (internal.printOptions.pageTitle) 
		{
			printWin.document.title = internal.printOptions.pageTitle;
		}
	
		// ...orientation...
		if (internal.printOptions.pageOrientation)
		{
			var paper = printWin.document.getElementById('paper');
			paper.className += ' ' + internal.printOptions.pageOrientation;
		}
		
		// ... and print note
		if (internal.printOptions.printNote)
		{
			var note = printWin.document.getElementById('printNote');
			note.innerHTML = internal.printOptions.printNote;
		}
		
		// Get the HTML from the <div> tag to be printed
		var printElement = Doc.element(internal.printOptions.printElementId);
		var printHtml = printElement.innerHTML;
		// Replace all <canvas> elements with <img>
		// since that seems the only way to make things work in IE
		printHtml = printHtml.replace(/\<canvas.*?\<\/canvas\>/g, '<img class="canvas-copy">');
	
		// Copy the HTML to the print-area in the print-window
		var printArea = printWin.document.getElementById('printarea');
		printArea.innerHTML = printHtml;
	
		// Copy the <canvas> elements as images to the <img> elements
		// using toDataURL()
		var srcCanvas = printElement.getElementsByTagName('canvas');
		var printImg = printArea.getElementsByClassName('canvas-copy');
		for (var i = 0; i < srcCanvas.length; i++)
		{
			printImg[i].src = srcCanvas[i].toDataURL();
		}
		
	}
	
	return reggie;
}();




var Wizard = function()
{
	var wizard = {};
	var internal = {};
	var debug = 1;
	
	var currentStep = 1;
	
	/**
		Validate the 'date'-input field that is the target of this event
		The date is optional unless 'required' is part if it's className.
		Dates should be either 4 or 8 digits with month+day or year+month+day.
		If 'data-allow-six-digits' is set on the target element, the year
		is allowed to be only 2 digits.
		
		4-digit (and 6-digit) dates auto-padded to 8 digits.
		
		The 'data-valid' attribute is set to 1 or 0 depending on the outcome,
		and 'data-warning' is set if the date is missing on non-required fields.
	*/
	wizard.validateDate = function(event)
	{
		var target = event.currentTarget;
		var isValid = false;
		var isWarning = false;
		
		// Reset current status
		wizard.setInputStatus(target);
		
		var date = target.value;
		if (date == '')
		{
			// No date, is this an error or a warning?
			var optional = target.className.indexOf('required') == -1;
			wizard.setInputStatus(target, optional ? 'warning' : 'invalid', 'Missing');
			isValid = optional;
			isWarning = optional;
		}
		else
		{
			var allowSixDigits = Data.int(target, 'allow-six-digits');
			date = Reggie.autoFillDate(date, allowSixDigits);
			target.value = date;
			
			if (!Dates.isDate(date, 'yyyyMMdd'))
			{
				Wizard.setInputStatus(target, 'invalid', 'Not a valid date');
			}
			else
			{
				Wizard.setInputStatus(target, 'valid');
				isValid = true;
			}
		}

		Data.set(target, 'valid', isValid ? 1 : 0);
		Data.set(target, 'warning', isWarning ? 1 : 0);
		
		if (!isValid && target.focus) target.focus();
	}

	
	wizard.isValid = function(element)
	{
		return Data.int(element, 'valid');
	}

	/**
		Change display status for an input field.
		@param prefix ID prefix to locate tags
		@param clazz The message clazz (valid, invalid, warning) or empty
		@param message The message (may be empty)
	*/
	wizard.setInputStatus = function(prefix, clazz, message)
	{
		if (prefix.id) prefix = prefix.id;
		
		var statusTag = Doc.element(prefix + '.status');
		statusTag.className = 'status ' + (clazz || '');
		
		var trTag = statusTag.parentNode;
		var trClass = Data.get(trTag, 'original-class');
		if (!trClass)
		{
			trClass = trTag.className || 'foo';
			Data.set(trTag, 'original-class', trClass);
		}
		trTag.className = trClass + ' ' + (clazz || '');
		
		var msgTag = Doc.element(prefix + '.message');
		if (msgTag)
		{
			if (message)
			{
				msgTag.innerHTML = message;
				Doc.show(msgTag, 'inline');
			}
			else
			{
				msgTag.innerHTML = message;
				Doc.hide(msgTag);
			}
		}
		else
		{
			statusTag.title = message || '';
		}
	}

	
	/**
		Event handler for input fields, that go to the
		next wizard step if TAB or ENTER key is pressed.
		A short delay is used to allow other event handler
		to run first (eg. for validation).
	*/
	wizard.goNextOnTabOrEnter = function(event)
	{
		if (event.keyCode == 9 || event.keyCode == 13) 
		{
			setTimeout(function() { wizard.goNext(true) }, 200);
		}
	}
	
	/**
		Event handler for input fields, that changes
		focus to the next element. The next element
		should be defined by 'data-next-focus' attribute.
		It is recommended that this is consisistent with TAB.
	*/
	wizard.focusOnEnter = function(event)
	{
		if (event.keyCode == 13)
		{
			var nextFocus = Doc.element(Data.get(event.currentTarget, 'next-focus'));
			if (nextFocus && nextFocus.focus) nextFocus.focus();
		}
	}
	
	// Event handler for the 'Next' navigation button
	wizard.goNextOnClick = function(event)
	{
		wizard.goNext(false);
	}
	
	/**
		Go to the next step in the wizard. This method will first
		submit 'wizard-validate' event to the current step. If at 
		least one event handler calls 'Event.preventDefault()' the 
		processing is aborted. 
		
		Otherwise, the current step is disabled and the 'wizard-initialize' 
		event is sent to the next step. The event handler is responsible
		for calling 'Wizard.setCurrentStep' to display the next step.
	*/
	wizard.goNext = function(auto)
	{
		var verify = Doc.element('verifyGoNext');
		if (verify && !verify.checked) return;
		
		var details = { 'auto': auto };
		
		// Send 'wizard-validate' event to current step
		var step = Doc.element('step-'+currentStep);
		var evt = document.createEvent('CustomEvent');
		evt.initCustomEvent('wizard-validate', true, true, details);
		if (!step.dispatchEvent(evt)) 
		{
			Doc.addClass('wizard', 'flash-error');
			setTimeout(internal.stopFlashing, 2000);
			return;
		}

		// Hide all navigation buttons and disable form controls in current step
		internal.hideButtons('navigation');
		internal.disableStep(step);
		
		// Send 'wizard-initialize' to the next step
		var nextStep = Doc.element('step-'+(currentStep+1));
		var evt = document.createEvent('CustomEvent');
		evt.initCustomEvent('wizard-initialize', true, true, details);
		nextStep.dispatchEvent(evt);
	}
	
	internal.stopFlashing = function()
	{
		Doc.removeClass('wizard', 'flash-error');
	}
	
	/**
		Finalize the registration of the wizard. This method will
		first submit 'wizard-validate' to the current step. If at 
		least one event handler calls 'Event.preventDefault()' the 
		processing is aborted.
		
		Otherwise, the current step is disabled and the 'wizard-submit' 
		event is sent to the 'wizard'. The event handler is responsible
		for finalizing the registration and displaying a proper
		message, for example, by calling Wizard.showFinalMessage()
	*/
	wizard.goRegister = function()
	{
		// Check if verification is enabled
		var verify = Doc.element('verifyGoNext');
		if (verify && !verify.checked) return;
		
		// Send 'wizard-validate' event to current step
		var step = Doc.element('step-'+currentStep);
		var evt = document.createEvent('Event');
		evt.initEvent('wizard-validate', false, true);
		if (!step.dispatchEvent(evt)) 
		{
			Doc.addClass('wizard', 'flash-error');
			setTimeout(internal.stopFlashing, 2000);
			return;
		}

		internal.hideButtons('navigation');
		internal.disableStep(step);
		
		// Send 'wizard-submit' to the next step
		var wiz = Doc.element('wizard');
		var evt = document.createEvent('Event');
		evt.initEvent('wizard-submit', false, true);
		wiz.dispatchEvent(evt);
	}
	
	// Ask the user for confirmation and then restart the wizard
	wizard.cancelWizard = function()
	{
		if (confirm('Cancel this registration?'))
		{
			wizard.restartWizard();
		}
	}
	
	// Restart the wizard without asking the user for confirmation
	wizard.restartWizard = function()
	{
		var url = location.href;
		if (url.indexOf('&restart=1') == -1) url += '&restart=1';
		location.href = url;
	}
	
	wizard.setCurrentStep = function(stepNo)
	{
		currentStep = stepNo;
		internal.enableStep(Doc.element('step-'+currentStep));
	}

	internal.disableStep = function(step)
	{
		step = Doc.element(step);
		internal.disableAllFormElements(step);
		Doc.addClass(step, 'disabled');
		Doc.hide('gonext-message');
		var stepNo = step.getElementsByClassName('step-no')[0];
		Events.addEventHandler(stepNo, 'click', internal.togglePastStep, {'step-id': step.id});
		stepNo.title = 'Show/hide this section';
	}
	
	internal.enableStep = function(step)
	{
		step = Doc.element(step);
		Doc.removeClass(step, 'disabled');
		Doc.show(step);
	}

	// Toggle visibility of a past step
	// The step is hidden if the 'auto-hide' class is present
	internal.togglePastStep = function(event)
	{
		var step = Data.get(event.currentTarget, 'step-id');
		Doc.addOrRemoveClass(step, 'auto-hide');
	}
	
	wizard.syncJsonRequest = function(url, method, postdata)
	{
		if (debug)
		{
			App.debug((method || 'GET') + ': ' + url);
			if (postdata) App.debug(postdata);
		}
		var request = Ajax.getXmlHttpRequest();
		request.open(method || 'GET', url, false);
		if (postdata) 
		{
			request.setRequestHeader("Content-Type", "application/json");
		}
		request.send(postdata || null);
		
		if (debug) App.debug(request.responseText);
		Wizard.hideWizardStatus();

		var response = null;
		var errorMsg = null;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				errorMsg = response.message;
			}
		}
		catch (err)
		{
			errorMsg = err.toString ? err.toString() : err;
		}
		
		if (errorMsg)
		{
			Wizard.setFatalError(errorMsg);
			return null;
		}
		else
		{
			return response;
		}
	}
	
	/**
		Submit an ansynchronous request for JSON data. 'GET' method
		is used if 'method' parameter is not set.
		The 'callback' should be a method accepting a single
		JSON parameter object. If the 'status' of the response
		is not 'ok' this method will call setFatalError and
		without calling the callback method.
	*/
	wizard.asyncJsonRequest = function(url, callback, method, postdata)
	{
		if (debug)
		{
			App.debug((method || 'GET') + ': ' + url);
			if (postdata) App.debug(postdata);
		}
		var request = Ajax.getXmlHttpRequest();
		request.jsonCallback = callback;
		request.open(method || 'GET', url, true);
		if (postdata) 
		{
			request.setRequestHeader("Content-Type", "application/json");
		}
		Ajax.setReadyStateHandler(request, internal.asyncCallbackWrapper, internal.asyncCallbackWrapper);
		request.send(postdata || null);
	}
	
	/**
		Callback wrapper for ansynchronous requests.
	*/
	internal.asyncCallbackWrapper = function(request)
	{
		if (debug) App.debug(request.responseText);
		Wizard.hideWizardStatus();

		var response = null;
		try
		{
			if (request.status != 200)
			{
				throw new Error(request.responseText);
			}
			else
			{
				if (request.getResponseHeader('Content-Type').indexOf('json') != -1)
				{
					response = JSON.parse(request.responseText);
					if (response.status != 'ok')
					{
						throw new Error(response.message || response.stacktrace);
					}
				}
				else
				{
					response = request.responseText;
				}
				request.jsonCallback.call(null, response);
			}
		}
		catch (err)
		{
			Wizard.setFatalError(Strings.encodeTags(err.toString ? err.toString(): err));
		}
	}

	
	/**
		A fatal error has occurred and the wizard will not be able to continue.
		This method will display the error message, disable all form input fields
		and buttons (except the 'Restart' button).
	*/
	wizard.setFatalError = function(msg)
	{
		App.debug(msg);
		internal.setWizardStatus('messagecontainer error', msg || 'Unexpected failure!');
		Doc.addClass('wizard', 'disabled');
		internal.disableAllFormElements('wizard', true);
		Doc.show('gorestart');
	}

	
	/**
		Display a 'loading' global wizard-status message.
	*/
	wizard.showLoadingAnimation = function(msg)
	{
		internal.setWizardStatus('loading', msg  || 'Working, please wait...');
	}

	wizard.showStatusMessage = function(msg)
	{
		App.debug(msg);
		internal.setWizardStatus('messagecontainer note', msg);
	}
	
	wizard.showGoNextConfirmation = function(verify, msg)
	{
		var check = '';
		if (verify)
		{
			check = '<input type="checkbox" name="verifyGoNext" id="verifyGoNext">';
			Doc.addClass('gonext', 'disabled');
			Doc.addClass('goregister', 'disabled');
		}
		
		var gonext = Doc.element('gonext-message');
		gonext.innerHTML = check + msg;
		Doc.show(gonext);
		
		if (verify)
		{
			Events.addEventHandler('verifyGoNext', 'click', internal.verifyInternalOnClick);
		}
	}
	
	wizard.hideGoNextConfirmation = function()
	{
		var gonext = Doc.element('gonext-message');
		gonext.innerHTML = '';
		Doc.hide(gonext);
		Doc.removeClass('gonext', 'disabled');
		Doc.removeClass('goregister', 'disabled');
	}
	
	internal.verifyInternalOnClick = function(event)
	{
		var frm = document.forms['reggie'];
		var checked = event.currentTarget.checked;
		Doc.addOrRemoveClass('goregister', 'disabled', !checked);
		Doc.addOrRemoveClass('gonext', 'disabled', !checked);
	}
	
	/**
		Show final registration messages in a list.
	*/
	wizard.showFinalMessage = function(messages)
	{
		var msg = '<ul>';
		for (var i = 0; i < messages.length; i++)
		{
			var msgLine = messages[i];
			if (msgLine.indexOf('[Warning]') >= 0)
			{
				msg += '<li class="warning">' + Strings.encodeTags(msgLine.replace('[Warning]', ''));
			}
			else
			{
				msg += '<li class="ok">' + Strings.encodeTags(msgLine);
			}
		}
		msg += '</ul>';
		internal.setWizardStatus('success', msg);
	}
	
	/**
		Hide the current global wizard-status message.
	*/
	wizard.hideWizardStatus = function()
	{
		Doc.hide('wizard-status');
	}
		
	/**
		Initialize the wizard:
		
		* Check that a project is active
	*/
	internal.initWizard = function()
	{
		// Check if project is enabled
		var wiz = Doc.element('wizard');
		if (wiz)
		{
			if (!App.getActiveProjectId())
			{
				wizard.setFatalError('No project has been selected.');
			}
		}
	}
	
	// Initialize some event handlers
	internal.initElements = function(element, autoInit)
	{
		if (autoInit == 'focus-on-enter')
		{
			Events.addEventHandler(element, 'keypress', wizard.focusOnEnter);
		}
	}
	
	/**
		Display a global wizard-status message. This may for example be
		an error message, a work-in-progress indicator, etc.
	*/
	internal.setWizardStatus = function(className, msg)
	{
		var status = Doc.element('wizard-status');
		status.className = className;
		status.innerHTML = msg;
		Doc.show(status);
	}

	// Disable all form elements inside the given container
	internal.disableAllFormElements = function(container, hideButtons)
	{
		container = Doc.element(container);
		var input = container.getElementsByTagName('input');
		for (var i = 0; i < input.length; i++)
		{
			input[i].disabled = true;
		}
		var select = container.getElementsByTagName('select');
		for (var i = 0; i < select.length; i++)
		{
			select[i].disabled = true;
		}
		var text = container.getElementsByTagName('textarea');
		for (var i = 0; i < text.length; i++)
		{
			text[i].disabled = true;
		}
		if (hideButtons)
		{
			internal.hideButtons(container);
		}
	}

	// Hide all 'basicbuttons':s in the given container
	internal.hideButtons = function(container)
	{
		container = Doc.element(container);
		var buttons = container.getElementsByClassName('basicbutton');
		for (var i = 0; i < buttons.length; i++)
		{
			Doc.hide(buttons[i]);
		}
	}

	Doc.addFinalizer(internal.initWizard);
	Doc.addElementInitializer(internal.initElements);
	return wizard;
}();

